#ifndef OSA_LINUX_CPP_RTL_H
#define OSA_LINUX_CPP_RTL_H

#include "../OsaRtl.h"

/******************************************************************************
******************************************************************************/
class OsaSpinLockImpl {
public:
  OsaSpinLockImpl()
  {
    spin_lock_init(&m_tSpinLock);
    m_nSavedFlags = 0xabcdef12;
    m_bLocked = false;
  }

  ~OsaSpinLockImpl()
  {
    if (m_bLocked) {
      Unlock();
    }
  }

  void Lock()   
  {
    __u32 flags;
    spin_lock_irqsave(&m_tSpinLock, flags);
    m_bLocked = true;
    m_nSavedFlags = flags;  
  }
  
  void Unlock()
  {
    if (m_bLocked) {
      spin_unlock_irqrestore(&m_tSpinLock, m_nSavedFlags);
    } else {
      OSA_EPRINTF(("OsaLinux: Attempt to unlock a non-locked spinlock\n"));
    }
  }

  bool IsLocked()
  {
    return m_bLocked;
  }

private:
  spinlock_t m_tSpinLock;
  __u32      m_nSavedFlags;
  bool       m_bLocked;
};


/******************************************************************************
******************************************************************************/
class OsaEventImpl {
public:
  OsaEventImpl()
    : m_pWaitQueue(NULL)
    , m_nEventSet(0)
  {}

  ~OsaEventImpl()
  {
    // Release possible waiting threads before destroying
    // This is an arbitrary decision on my part /LME990923
    SetEvent();
  }

  bool SetEvent()
  {
    int nEventSetBefore = m_nEventSet; 
    m_nEventSet = 1;
    osalinux_wake_up(&m_pWaitQueue);
    return nEventSetBefore ? true : false;
  }

  void ClearEvent()
  {
    m_nEventSet = 0;
  }

private:
  friend int OsaWaitForSingleEvent(OsaEventImpl *pEvent, OsaUint32 nWaitMaxMs);
  struct wait_queue *m_pWaitQueue;
  OsaUint32         m_nEventSet;
  OsaSpinLockImpl   m_access;
};


/******************************************************************************
  A Timeout of zero allows the testing of a set of wait conditions and for 
  conditionally performing any side effects if the wait can be immediately
  satisfied
******************************************************************************/
int OsaWaitForSingleEvent(OsaEventImpl *pEvent, OsaUint32 nWaitMaxMs);

#endif
